#include "zddice.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <time.h>

/* Global results buffer */
static QString rbuf = "";

/* 
 *  Constructs a zddice which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
zddice::zddice( QWidget* parent,  const char* name, WFlags fl )
    : zddiceBase( parent, name, fl )
{
  connect(quit, SIGNAL(clicked()), this, SLOT(GoodBye()));
  connect(help, SIGNAL(clicked()), this, SLOT(Help()));
  connect(reset, SIGNAL(clicked()), this, SLOT(Reset()));
  connect(d4, SIGNAL(pressed()), this, SLOT(Roll()));
  connect(d6, SIGNAL(pressed()), this, SLOT(Roll()));
  connect(d8, SIGNAL(pressed()), this, SLOT(Roll()));
  connect(d10, SIGNAL(pressed()), this, SLOT(Roll()));
  connect(d12, SIGNAL(pressed()), this, SLOT(Roll()));
  connect(d20, SIGNAL(pressed()), this, SLOT(Roll()));
  connect(d30, SIGNAL(pressed()), this, SLOT(Roll()));
  connect(d100, SIGNAL(pressed()), this, SLOT(Roll()));
  /* set random initial seed - ONLY ONCE! */
  srand( (unsigned)time( NULL ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
zddice::~zddice()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Help...
 */
                                                                               
void zddice::Help()
{
  QMessageBox::information( this, "zD20 Dice - Help",
                            "Zaurus D20 Gaming Dice\n\n"
                            "Select the quantity of dice,\n"
                            "select the +/- modifier,\n"
                            "click the dice image.\n"
                            "The bones are rolled &\n"
                            "the result is displayed!\n"
                            "Click reset to clear the\n"
                            "results window.\n\n"
                            "License: GPL\n\n"
                            "Written by David Brown\n"
                            "<db@cinlug.org>." );
}

/*
 *  Reset...
 */
void zddice::Reset()
{
  dresults->setText( tr( "" ) );
  dnum->setValue( 1 );
  dmod->setValue( 0 );
  if( !dplus->isOn() )
    dplus->toggle();
  rbuf = "";
}

/*
 *  Reset...
 */
void zddice::Roll()
{
  int inx, num, mod, type, mtype, roll, total;
  char ctype;
  char buf1[512];
  char buf2[512];
  char tmp2[16];
  QString tmp;

  num = dnum->value();
  mod = dmod->value();
  if( dplus->isOn() ) {
    mtype = 1;
    ctype = '+';
  }
  else {
    mtype = -1;
    ctype = '-';
  }

  if( d4->isDown() )
    type = 4;
  else if( d6->isDown() )
    type = 6;
  else if( d8->isDown() )
    type = 8;
  else if( d10->isDown() )
    type = 10;
  else if( d12->isDown() )
    type = 12;
  else if( d20->isDown() )
    type = 20;
  else if( d30->isDown() )
    type = 30;
  else 
    type = 100;

  /* Jumbled mismatch of C and C++ strings...doh! */
  total = 0;
  strcpy( buf2, "( " );
  for( inx=0; inx<num; inx++ ) {
    roll = (int)(rand() / (((double)RAND_MAX + 1)/type))+1;
    sprintf( tmp2, "%d", roll );
    strcat( buf2, tmp2 );
    strcat( buf2, " " );
    total += roll;
  }
  strcat( buf2, ")" );
  if( mod ) {
  sprintf( buf1, "%d = %dd%d%c%d %s",
                 total+(mtype*mod), num, type, ctype, mod, buf2 );
  }
  else {
  sprintf( buf1, "%d = %dd%d %s", total, num, type, buf2 );
  }

  tmp = rbuf;
  rbuf = buf1;
  rbuf += '\n';
  rbuf += tmp;

  dresults->setText( rbuf );
}

/*
 *  Quit...
 */
void zddice::GoodBye()
{
  close();
}
